/*
 * Copyright (c) 2024 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

/**
 * @file doca_dpa_dev_prdma.h
 * @page doca_dpa_prdma
 * @defgroup DOCA_DPA_DEVICE_PRDMA DOCA DPA Device - PRDMA
 * @ingroup DOCA_DPA_DEVICE
 * DOCA DPA Device - PRDMA
 * @{
 */

#ifndef DOCA_DPA_DEV_PRDMA_H_
#define DOCA_DPA_DEV_PRDMA_H_

#include <doca_dpa_dev.h>
#include <doca_dpa_dev_buf.h>

#ifdef __cplusplus
extern "C" {
#endif

/**
 * @brief DPA PRDMA handle type definition
 */
__dpa_global__ typedef uint64_t doca_dpa_dev_prdma_t;

/**
 * @brief DPA PRDMA Poster DPA handle type definition
 */
__dpa_global__ typedef uint64_t doca_dpa_dev_prdma_poster_t;

/**
 * @brief Ack PRDMA receive operations to enable reposting the buffers
 *
 * @param[in] prdma - PRDMA DPA handle
 * @param[in] num_acked - Number of receive operations to ack
 */
DOCA_EXPERIMENTAL
void doca_dpa_dev_prdma_connection_receive_ack(doca_dpa_dev_prdma_t prdma, uint32_t num_acked);

/**
 * @brief Send completions to the user QP
 *
 * @param[in] prdma_poster - PRDMA DPA Poster QP handle
 * @param[in] completion_element - pointer to the CQE to send to the user
 * @param[in] mmap_handle - send DOCA Mmap handle
 * @param[in] flags - bitwise OR of enum doca_dpa_dev_submit_flag (see enum doca_dpa_dev_submit_flag for more details)
 */
DOCA_EXPERIMENTAL
void doca_dpa_dev_prdma_poster_post(doca_dpa_dev_prdma_poster_t prdma_poster,
				    doca_dpa_dev_completion_element_t completion_element,
				    doca_dpa_dev_mmap_t mmap_handle,
				    uint32_t flags);
#ifdef __cplusplus
}
#endif

/** @} */

#endif /* DOCA_DPA_DEV_PRDMA_H_ */
